package com.yilesoft.app.textimage.moveedit;

import android.content.Context;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.graphics.Typeface;
import android.os.Build;
import androidx.annotation.RequiresApi;
import android.text.Editable;
import android.text.InputFilter;
import android.text.SpannableString;
import android.text.SpannableStringBuilder;
import android.text.Spanned;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.text.style.AbsoluteSizeSpan;
import android.text.style.BackgroundColorSpan;
import android.text.style.ForegroundColorSpan;
import android.text.style.StrikethroughSpan;
import android.text.style.StyleSpan;
import android.text.style.UnderlineSpan;
import android.util.AttributeSet;
import android.view.Gravity;
import android.view.MotionEvent;


import com.just.wholewriter.json.Constant;
import com.share.cool.PixelUtil;
import com.yilesoft.app.textimage.R;
import com.yilesoft.app.textimage.util.SEditText;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;


public class CustomsEditText extends SEditText {

    private static final String TAG = CustomsEditText.class.getSimpleName();


    // Attributes
    private Paint testPaint;

    private int type = -1;
    public static final int TEXT_SIZE = 1;
    public static final int TEXT_COLOR = 2;
    public static final int TEXT_STYLE = 3;
    private SpansInfo sizeInfo;
    private SpansInfo colorInfo;
    private SpansInfo styleInfo;
    private List<SpansInfo> spansInfoList;
    private float mDefaultTextSize = 0;//文字初始大小 sp
    private float mTotalTextSize = 0;//文本框整体文字大小 px
    private boolean isAdd = false;//是否是加大字体
    private int mLastTotalStyle = 0;//字体整体typeface 0:正常 1：加粗
    private final int MAX_TEXT_SIZE = 72;//sp
    public static final int MIN_TEXT_SIZE = 10;//sp
    public static final int MAX_DRAG_TEXT_SIZE = 30;//sp 最大拖动放大size
    private  int ONE_SP ;//单位px
    private float oriRatio = 1.0f;//初始 字号/宽度 比值
    private float rTop = 1.0f;//上间距与高度的比值
    private float rBottom = 1.0f;//下间距与高度的比值
    private float rLeft = 1.0f;//左间距与高度的比值
    private float rRight = 1.0f;//右间距与高度的比值
    private float perSizeWidth = 0;
    private int mCurrentSelection = 0;//当前光标序号
    private final int BUBBLE_MAX_TEXT_SIZE = 17;//单位 sp ，气泡文本框拖动缩放字体最大值
    private boolean isBubble = false;//是否为气泡
    private BubbleInfo bubbleInfo;
    private boolean onTextChangedError = false;
    private final int MAX_LENGTH = 300;//限制文本框最多300个字
    private boolean isCopy = false;
    private boolean isOpenAutoFit = false;//默认关掉文本框根据文字自适应宽高
    private TextTypeBean textTypeBean;
    private EditFrameInfo editFrameInfo;
    private Context context;
    private int versionCode = 0;//版本兼容用 ： 前版本是 0 大众版是 1

    //新版本文字效果参数：
    private Effects effects;
    private SpannableStringBuilder spannableStringBuilder;
    private boolean isPreview = false;
    private int fontId;

    public CustomsEditText(Context context) {
            super(context);
        initialise(context);
    }

    public CustomsEditText(Context context, AttributeSet attrs) {
        super(context, attrs);
        initialise(context);
    }

    public CustomsEditText(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);

        initialise(context);
    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {

        return isEnabled() && super.onTouchEvent(event) && !isPreview;
    }

    private void initialise(Context context) {

        this.context=context;
        ONE_SP = PixelUtil.sp2px(1,getContext());//单位px
        testPaint = new Paint();
        testPaint.set(this.getPaint()); // 获取模拟的paint
        setVersionCode(Utils.getTextVersionCode(getContext()));
        switch (versionCode) {
            case 0:
                setFilters(new InputFilter[]{new InputFilter.LengthFilter(MAX_LENGTH)});
                spansInfoList = new ArrayList<>();
                addTextChangedListener(mTextWatcher);
                break;
            case 1:
                setLineSpacing(0, 1.5f);//默认1.5倍行间距
                spannableStringBuilder = new SpannableStringBuilder();
                effects = getDefaultEffects();
                if (isBubble) {
                    setFilters(new InputFilter[]{new InputFilter.LengthFilter(MAX_LENGTH)});
                    spansInfoList = new ArrayList<>();
                    addTextChangedListener(mTextWatcher);
                }
                break;
        }
    }

    @Override
    protected void onFocusChanged(boolean focused, int direction, Rect previouslyFocusedRect) {
        super.onFocusChanged(focused, direction, previouslyFocusedRect);
        if (!focused && getText() != null) {
            setSelection(getText().length());
        }
    }

    /**
     * 获取当前文字总体大小 !注意单位 px!
     *
     * @return 当前字体总体大小，单位px
     */
    @Override
    public float getTextSize() {
        if (versionCode == 1) {
            return super.getTextSize();
        }
        if (mTotalTextSize == 0) {
            return super.getTextSize();
        } else {
            return mTotalTextSize;
        }
    }

    /**
     * 根据控件宽高自适应文字大小
     */
    private void refitText(String text, int width) {
        int currentLineCount = getLineCount() == 0 ? (int) Math.ceil((length() * getTextSize()) / getWidth()) : getLineCount();
        int lineHeight = getLineHeight();
        int textTotalHeight = currentLineCount * lineHeight;
        if (width > 0) {
            int availableWidth = width - this.getPaddingLeft()
                    - this.getPaddingRight();// 获取改TextView的画布可用大小
            int availableHeight = getHeight() - (getPaddingBottom() + getPaddingTop());
            float trySize = getTextSize();
            Paint tempPaint = new Paint();
            tempPaint.setTextSize(trySize);
            if ((trySize > PixelUtil.sp2px(MIN_TEXT_SIZE,context))
                    && ((tempPaint.measureText(text) > availableWidth) && textTotalHeight > availableHeight)) {//减小字体
                do {
                    //减小字体
                    trySize -= ONE_SP;
                    if (trySize < PixelUtil.sp2px(MIN_TEXT_SIZE,context)) {
                        trySize = PixelUtil.sp2px(MIN_TEXT_SIZE,context);
                    }
                    setTotalTextSize(PixelUtil.px2sp(trySize,context));
                    //获取字体宽度高度
                    tempPaint.setTextSize(trySize);
                    currentLineCount = getLineCount() == 0 ? (int) Math.ceil((length() * trySize) / getWidth()) : getLineCount();
                    int textHeight = (int) getFontHeight(tempPaint);
                    lineHeight = (int) (textHeight * getLineSpacingMultiplier() + (getLineSpacingExtra() == 0 ? 10 : getLineSpacingExtra()));
                    textTotalHeight = currentLineCount * lineHeight;
                } while (trySize > PixelUtil.sp2px(MIN_TEXT_SIZE,context) &&
                        ((tempPaint.measureText(text) > availableWidth) &&
                                textTotalHeight > availableHeight));//字体宽度>可用宽度 && 字体高度>可用高度
            } else {
                setTotalTextSize(PixelUtil.px2sp(trySize,context));
            }
        }
    }

    int lineCount;
    int textHeight;

    private TextWatcher mTextWatcher = new TextWatcher() {

        String lastOnText = "";
        String lastAfterText = "";

        @Override
        public void beforeTextChanged(CharSequence s, int start, int count, int after) {

        }

        @Override
        public void onTextChanged(CharSequence s, int start, int before, int count) {
            if (isBubble || versionCode == 0) {
                if (lastAfterText.equals(s.toString())) {
                    return;
                }
                lastAfterText = s.toString();
                mCurrentSelection = 0;
                LogUtil.d(TAG + " onTextChanged : start = " + start + " ,before = " + before + " ,count = " + count + " ,s = " + s.toString());
                if (Utils.isListEmpty(spansInfoList)) {//兼容老草稿
                    spansInfoList = new ArrayList<>();
                }
                //增加报错预防：遇到报错，清空当前文本，清空当前spansInfoList
                try {
                    if (before == 0 && count > 0) {//增加
                        for (int i = 0; i < count; i++) {//从start开始插入count个SpansInfo对象
                            spansInfoList.add(start + i, new SpansInfo());
                        }
                        mCurrentSelection = start + count;
                    } else if (before > 0 && count == 0) {//删除
                        //从start开始删除before个SpansInfo对象
                        if (Utils.isListEmpty(spansInfoList)) {
                            return;
                        }
                        List<SpansInfo> tempList = new ArrayList<>(spansInfoList);
                        if (start == length() - 1) {
                            for (int i = 0; i < before; i++) {//从start开始删除before个SpansInfo对象
                                spansInfoList.remove(tempList.get(start - i));
                            }
                        } else {
                            for (int i = 0; i < before; i++) {
                                spansInfoList.remove(tempList.get(start + i));
                            }
                        }
                        mCurrentSelection = start - before + 1;
                    } else if (before > 0 && count > 0) {//适配搜狗输入法
                        if (before < count) {
                            for (int i = 0; i < count - before; i++) {//从start开始插入count-before个SpansInfo对象
                                spansInfoList.add(start + i, new SpansInfo());
                            }
                            mCurrentSelection = start + count;
                        } else if (before > count) {//从start开始删除before-count个SpansInfo对象
                            //适配百度输入法华为版
                            if (before - count > 25 && count == 1) {
                                for (int i = before - count + start; i > start; i--) {
                                    spansInfoList.remove(i);
                                }
                                mCurrentSelection = start + 1;
                            } else {
                                for (int i = 0; i < before - count; i++) {
                                    spansInfoList.remove(start - i);
                                }
                                mCurrentSelection = start + count;
                            }
                        } else {
                            mCurrentSelection = before;
                        }
                    }
                } catch (Exception e) {
                    LogUtil.e(TAG + " onTextChanged error : " + e.toString());
                    e.printStackTrace();
                    setText("");
                    spansInfoList.clear();
                    onTextChangedError = true;
                }
                if (mCurrentSelection < 0) {
                    mCurrentSelection = 0;
                } else if (mCurrentSelection > length()) {
                    mCurrentSelection = length();
                }
                LogUtil.d(TAG + " onTextChanged : mCurrentSelection = " + mCurrentSelection);
            }
        }

        @Override
        public void afterTextChanged(Editable s) {
            if (versionCode == 0 || isBubble) {
                if (lastOnText.equals(s.toString()) || onTextChangedError) {
                    if (onTextChangedError) {
                        onTextChangedError = false;
                    }
                    return;
                }
                lastOnText = s.toString();
                fitCurrentTypeFace();
                if (isOpenAutoFit || isBubble) {
                    refitText(s.toString(), getWidth());
                    int currentLineCount = getLineCount() == 0 ? (int) Math.ceil((length() * getTextSize()) / getWidth()) : getLineCount();
                    int lineHeight = getLineHeight();
                    if (lineCount != currentLineCount && getTextSize() <= PixelUtil.sp2px(MIN_TEXT_SIZE,context)) {//字体调整到最小后才增加高度
                        lineCount = currentLineCount;
                        textHeight = lineHeight * lineCount;
                        if (textHeight != getHeight() && textAutoHeight != null) {
                            textAutoHeight.refreshHeight(textHeight);
                        }
                    }
                } else {
                    setTotalTextSize(PixelUtil.px2sp(mTotalTextSize,context));
                }
            }
        }
    };

    /**
     * 获取当前文本高度
     */
    @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
    public int getTextHeight() {
        int currentLineCount = 0;
        if (getLineCount() == 0 && getWidth() == 0) {
            int maxAvailableWidth = (int) (PixelUtil.getScreenWH(context)[0] - PixelUtil.dp2Pixel(15,context) + 30 -PixelUtil.getScreenWH(context)[0]/4);//减去的整数见MyEditText中的addEditText（）方法
            Paint paint = new Paint();
            paint.setTextSize(getTextSize());
            paint.setLetterSpacing(getLetterSpacing());
            if (getText() != null) {
                currentLineCount = (int) Math.ceil(paint.measureText(getText().toString()) / maxAvailableWidth);
            }
        } else {
            currentLineCount = (getLineCount() == 0 ? (int) Math.ceil((length() * getTextSize()) / getWidth()) : getLineCount());
        }
        int lineHeight = getLineHeight();
        int textHeight = lineHeight * currentLineCount - (lineHeight/3);
        return (int) (textHeight + PixelUtil.dp2Pixel(10,context));
    }

    /**
     * 获取当前文本宽度
     */
    @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
    public int getTextWidth() {
        int textWidth = getMinWidth();
        Paint paint = new Paint();
        paint.setTypeface(getTypeface());
       // paint.setTypeface(FontUtils.getTypeFace(effects.getFontId()));
        paint.setTextSize(getTextSize());
        paint.setLetterSpacing(getLineSpacingExtra());
        if (getText() != null) {
            textWidth = (int) paint.measureText(getText().toString());
        }
        return (int) (textWidth+PixelUtil.dp2Pixel(20,context));
    }

    /**
     * 获取单个字的宽度
     */
    @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
    public int getSingleTextWidth() {
        int textWidth = getMinWidth();
        Paint paint = new Paint();
        paint.setTextSize(getTextSize());
        paint.setLetterSpacing(getLineSpacingExtra());
        LogUtil.i(TAG + " getTextWidth : textSize = " + getTextSize() + " ,lineSpacing = " + getLineSpacingExtra());
        if (getText() != null) {
            textWidth = (int) paint.measureText("我");//随便一个字都行
        }
        return textWidth;
    }

    @Override
    public int getLineHeight() {
        Paint tempPaint = new Paint();
        float textSize = getTextSize();
        tempPaint.setTextSize(textSize);
        int textHeight = (int) getFontHeight(tempPaint);
        return (int) (textHeight * getLineSpacingMultiplier() + (getLineSpacingExtra() == 0 ? 10 : getLineSpacingExtra()));
    }

    public float getFontHeight(Paint paint) {

        paint.setTypeface(getTypeface());
        //paint.setTypeface(FontUtils.getTypeFace(effects.getFontId()));
        Paint.FontMetrics fm = paint.getFontMetrics();
        //文字基准线的下部距离-文字基准线的上部距离 = 文字高度
        float fontHeight = fm.descent - fm.ascent;
        return fontHeight;
    }

    @Override
    protected void onSizeChanged(int w, int h, int oldw, int oldh) {
        switch (versionCode) {
            case 0:
                /*1.计算原始比例 最小字号px/宽度 oriRatio
                 * 2.根据当前宽度计算出相应字号
                 * 3.设置为计算出的字号
                 */
                //1.计算原始比例 字号px/宽度 oriRatio
                if (oldw == 0 && w != 0) {
                    int width = getMinWidth() == 0 ? w : getMinWidth();
                    oriRatio = (float) PixelUtil.sp2px(MIN_TEXT_SIZE,context) / width;
                    perSizeWidth = (float) width / getTextSize();
                    if (bubbleInfo != null) {
                        rLeft = (float) bubbleInfo.getBg_left() / w;
                        rRight = (float) bubbleInfo.getBg_right() / w;
                    }
                }
                if (oldh == 0 && h != 0) {
                    if (bubbleInfo != null) {
                        rTop = (float) bubbleInfo.getBg_top() / h;
                        rBottom = (float) bubbleInfo.getBg_buttom() / h;
                    }
                }
                break;
            case 1:
                //do nothing
                break;
        }
    }

    /**
     * 直接设置字体大小
     *
     * @param size 单位sp
     */
    public void setTextSizeDirectly(float size) {
        if (getText() != null) {
            type = TEXT_SIZE;
            mCurrentSelection = getSelectionStart();
            int selectionEnd = getSelectionEnd();
            if (Utils.isListEmpty(spansInfoList)) {//兼容老草稿
                spansInfoList = new ArrayList<>();
                for (int i = 0; i < length(); i++) {
                    SpansInfo spansInfo = new SpansInfo();
                    spansInfoList.add(spansInfo);
                }
            }
            if (getSelectionStart() == getSelectionEnd()) {
                for (int i = 0; i < spansInfoList.size(); i++) {
                    SpansInfo tempSpans = spansInfoList.get(i);
                    Map<Integer, Integer> spansMap = tempSpans.getSpansInfoMap();
                    spansMap.put(TEXT_SIZE, PixelUtil.sp2px(size,context));
                }
                mTotalTextSize = PixelUtil.sp2px(size,context);
            } else {
                for (int i = getSelectionStart(); i < getSelectionEnd(); i++) {
                    SpansInfo tempSpans = spansInfoList.get(i);
                    Map<Integer, Integer> spansMap = tempSpans.getSpansInfoMap();
                    spansMap.put(TEXT_SIZE, PixelUtil.sp2px(size,context));
                }
            }
            setText(loadSpans());
            setSelection(mCurrentSelection, selectionEnd);
        }
    }

    /**
     * 设置整体字体大小，用于文本占用宽度大于当前文本框宽度时，缩小字体使用
     *
     * @param size 字体大小，单位sp
     */
    public void setTotalTextSize(float size) {
        if (mDefaultTextSize == 0) {
            mDefaultTextSize = size + 2;
        }
        if (getText() != null) {
            type = TEXT_SIZE;
            mCurrentSelection = getSelectionStart();
            if (Utils.isListEmpty(spansInfoList)) {//兼容老草稿
                spansInfoList = new ArrayList<>();
                for (int i = 0; i < length(); i++) {
                    SpansInfo spansInfo = new SpansInfo();
                    spansInfoList.add(spansInfo);
                }
            }
            for (int i = 0; i < spansInfoList.size(); i++) {
                SpansInfo tempSpans = spansInfoList.get(i);
                Map<Integer, Integer> spansMap = tempSpans.getSpansInfoMap();
                if (spansMap.get(TEXT_SIZE) == null || spansMap.get(TEXT_SIZE) == mTotalTextSize) {
                    spansMap.put(TEXT_SIZE, PixelUtil.sp2px(size,context));
                } else {
                    int oldSize = spansMap.get(TEXT_SIZE);
                    if (oldSize > PixelUtil.sp2px(size,context)) {//缩小
                        int newSize = oldSize - ONE_SP;
                        if (newSize < PixelUtil.sp2px(MIN_TEXT_SIZE,context)) {
                            newSize = PixelUtil.sp2px(MIN_TEXT_SIZE,context);
                        }
                        spansMap.put(TEXT_SIZE, newSize);
                    } else if (oldSize < PixelUtil.sp2px(size,context)) {//放大
                        int newSize = oldSize + ONE_SP;
                        if (newSize > PixelUtil.sp2px(MAX_TEXT_SIZE,context)) {
                            newSize = PixelUtil.sp2px(MAX_TEXT_SIZE,context);
                        }
                        spansMap.put(TEXT_SIZE, newSize);
                    } else {
                        spansMap.put(TEXT_SIZE, PixelUtil.sp2px(size,context));
                    }
                }
            }
            setText(loadSpans());
            setSelection(mCurrentSelection);
            if (mTotalTextSize == 0) {
                mTotalTextSize = PixelUtil.sp2px(size + 2,context);
            } else {
                mTotalTextSize = PixelUtil.sp2px(size,context);
            }
        }
    }

    /**
     * 整体字体尺寸加ds px
     */
    public void addDs(int ds) {
        if (getText() != null) {
            isAdd = true;
            if (isBubble) {
                if (getTextSize() + (ds == 1 ? 2 : ds) > PixelUtil.sp2px(BUBBLE_MAX_TEXT_SIZE,context)) {
                    setTotalTextSize(BUBBLE_MAX_TEXT_SIZE);
                } else {
                    setTotalTextSize(PixelUtil.px2sp(getTextSize() + (ds == 1 ? 2 : ds),context));
                }
            } else {
                if (getTextSize() + (ds == 1 ? 2 : ds) > PixelUtil.sp2px(MAX_DRAG_TEXT_SIZE,context)) {
                    setTotalTextSize(MAX_DRAG_TEXT_SIZE);
                } else {
                    setTotalTextSize(PixelUtil.px2sp(getTextSize() + (ds == 1 ? 2 : ds),context));
                }
            }
        }
    }

    /**
     * 整体字体尺寸减ds px
     */
    public void lessDs(int ds) {
        if (getText() != null) {
            if (getTextSize() - (ds == 1 ? 2 : ds) < PixelUtil.sp2px(MIN_TEXT_SIZE,context)) {
                setTotalTextSize(MIN_TEXT_SIZE);
            } else {
                setTotalTextSize(PixelUtil.px2sp(getTextSize() - (ds == 1 ? 2 : ds),context));
            }
        }
    }

    /**
     * ！注意单位！
     *
     * @param size 字体大小，单位sp
     */
    @Override
    public void setTextSize(float size) {
        if(versionCode == 1){
            super.setTextSize(size);
            effects.setTextSize((int) size);
        }else if(versionCode == 0 || isBubble){
            if (mDefaultTextSize == 0) {
                mDefaultTextSize = size + 2;
            }
            if (getText() != null) {
                type = TEXT_SIZE;
                mCurrentSelection = getSelectionStart();
                for (int i = 0; i < spansInfoList.size(); i++) {
                    SpansInfo tempSpans = spansInfoList.get(i);
                    Map<Integer, Integer> spansMap = tempSpans.getSpansInfoMap();
                    if (spansMap.get(TEXT_SIZE) == null) {
                        spansMap.put(TEXT_SIZE, PixelUtil.sp2px(size,context));
                    } else {
                        float oriSize = spansMap.get(TEXT_SIZE);
                        spansMap.remove(TEXT_SIZE);
                        if (isAdd) {
                            if (PixelUtil.px2sp(oriSize,context) < MAX_TEXT_SIZE) {
                                tempSpans.getSpansInfoMap().put(TEXT_SIZE, (int) oriSize + ONE_SP);
                            } else {
                                tempSpans.getSpansInfoMap().put(TEXT_SIZE, (int) oriSize);
                            }
                        } else {
                            if (PixelUtil.px2sp(oriSize,context) > MIN_TEXT_SIZE) {
                                tempSpans.getSpansInfoMap().put(TEXT_SIZE, (int) oriSize - ONE_SP);
                            } else {
                                tempSpans.getSpansInfoMap().put(TEXT_SIZE, (int) oriSize);
                            }
                        }
                    }
                }
                setText(loadSpans());
                setSelection(mCurrentSelection);
                if (mTotalTextSize == 0) {
                    mTotalTextSize = PixelUtil.sp2px(size + 2,context);
                } else {
                    mTotalTextSize = PixelUtil.sp2px(size,context);
                }
            }
            isAdd = false;
        }
    }

    //修改部分文字大小
    public void changeSomeWordsSize(float size) {
        if (getText() != null) {
            int start = getSelectionStart(), end = getSelectionEnd();
            type = TEXT_SIZE;
            filter();
            for (int i = 0; i < end - start; i++) {
                SpansInfo tempSpans = spansInfoList.get(start + i);
                tempSpans.getSpansInfoMap().put(TEXT_SIZE, PixelUtil.sp2px(size,context));
            }
            setText(loadSpans());
            setSelection(start, end);
        }
    }

    @Override
    public void setTextColor(int color) {
        if(versionCode == 1){
            super.setTextColor(color);
            effects.setTextColor(color);
        }else if(versionCode == 0 || isBubble){
            int start = getSelectionStart(), end = getSelectionEnd();
            type = TEXT_COLOR;
            if (start != end && getText() != null) {
                filter();
                if (Utils.isListEmpty(spansInfoList)) {
                    return;
                }
                for (int i = 0; i < end - start; i++) {
                    SpansInfo tempSpans = spansInfoList.get(start + i);
                    tempSpans.getSpansInfoMap().put(TEXT_COLOR, color);
                }
                setText(loadSpans());
                setSelection(start, end);
            } else {
                super.setTextColor(color);
            }
        }
    }

    //过滤
    private void filter() {
        if (getText() == null || Utils.isListEmpty(spansInfoList)) {
            return;
        }
        for (int i = 0; i < length(); i++) {
            SpansInfo tempSpans = spansInfoList.get(i);
            Map<Integer, Integer> spansMap = tempSpans.getSpansInfoMap();
            //如果是改变字体大小，过滤掉字体大小的span
            if (getSelectionStart() == getSelectionEnd() || (i >= getSelectionStart() && i < getSelectionEnd())) {
                if (type == TEXT_SIZE) {
                    spansMap.remove(TEXT_SIZE);
                } else if (type == TEXT_STYLE) {
                    spansMap.remove(TEXT_STYLE);
                }
            }
        }
        type = -1;
    }

    //加载已有的span
    private SpannableString loadSpans() {
        if (getText() == null || Utils.isListEmpty(spansInfoList)) {
            return null;
        }
        SpannableString ss = new SpannableString(Utils.getString(getText().toString()));
        LogUtil.d(TAG + " loadSpans :　ss = " + Utils.getString(getText().toString()) + " ,spansList size = " + spansInfoList.size());
        if (spansInfoList.size() > ss.length()) {
            LogUtil.e(TAG + " loadSpans : list size > ss.length, will result in IndexOutOfBoundsException");
            int deleteIndex = spansInfoList.size() - 1;
            do {
                spansInfoList.remove(deleteIndex--);
            } while (deleteIndex >= ss.length());
        }
        for (int i = 0; i < spansInfoList.size(); i++) {
            SpansInfo tempSpans = spansInfoList.get(i);
            Map<Integer, Integer> spansMap = tempSpans.getSpansInfoMap();
            for (Integer key : spansMap.keySet()) {
                switch (key) {
                    case TEXT_SIZE:
                        AbsoluteSizeSpan sizeSpan = new AbsoluteSizeSpan(spansMap.get(key));
                        ss.setSpan(sizeSpan, i, i + 1, Spanned.SPAN_INCLUSIVE_EXCLUSIVE);
                        break;
                    case TEXT_COLOR:
                        ForegroundColorSpan colorSpan = new ForegroundColorSpan(spansMap.get(key));
                        ss.setSpan(colorSpan, i, i + 1, Spanned.SPAN_INCLUSIVE_EXCLUSIVE);
                        break;
                    case TEXT_STYLE:
                        StyleSpan styleSpan = new StyleSpan(spansMap.get(key));
                        ss.setSpan(styleSpan, i, i + 1, Spanned.SPAN_INCLUSIVE_EXCLUSIVE);
                        break;
                }
            }
        }
        return ss;
    }

    /**
     * 新输入的文字适应当前TypeFace
     */
    private void fitCurrentTypeFace() {
        if (getText() == null || Utils.isListEmpty(spansInfoList) || isCopy) {
            if (isCopy) isCopy = false;
            return;
        }
        for (int i = 0; i < spansInfoList.size(); i++) {
            SpansInfo tempSpans = spansInfoList.get(i);
            Map<Integer, Integer> spansMap = tempSpans.getSpansInfoMap();
            if (spansMap.get(TEXT_STYLE) == null) {
                spansMap.put(TEXT_STYLE, mLastTotalStyle);
            }
        }
        setText(loadSpans());
        setSelection(mCurrentSelection);
    }

    @Override
    public void setTypeface(Typeface tf, int style) {
        if(versionCode == 1){
            super.setTypeface(tf, style);
        }else if(versionCode == 0 || isBubble){
            if (getText() == null || Utils.isListEmpty(spansInfoList)) {
                return;
            }
            type = TEXT_STYLE;
            mCurrentSelection = getSelectionStart();
            int start = getSelectionStart();
            int end = getSelectionEnd();
            if (start == end) {
                for (int i = 0; i < spansInfoList.size(); i++) {
                    SpansInfo tempSpans = spansInfoList.get(i);
                    Map<Integer, Integer> spansMap = tempSpans.getSpansInfoMap();
                    spansMap.remove(TEXT_STYLE);
                    spansMap.put(TEXT_STYLE, mLastTotalStyle == 1 ? 0 : 1);
                }
                mLastTotalStyle = mLastTotalStyle == 1 ? 0 : 1;
                setText(loadSpans());
                setSelection(mCurrentSelection);
            } else {
                for (int i = 0; i < end - start; i++) {
                    SpansInfo tempSpans = spansInfoList.get(start + i);
                    Map<Integer, Integer> spansMap = tempSpans.getSpansInfoMap();
                    if (spansMap.get(TEXT_STYLE) == null) {
                        tempSpans.getSpansInfoMap().put(TEXT_STYLE, Typeface.BOLD);
                    } else {
                        int oldStyle = spansMap.get(TEXT_STYLE);
                        tempSpans.getSpansInfoMap().put(TEXT_STYLE, oldStyle == Typeface.BOLD ? Typeface.NORMAL : Typeface.BOLD);
                    }
                }
                setText(loadSpans());
                setSelection(start, end);
            }
        }
    }

    /**
     * 填充字体效果
     */
    public void fillContent(EditFrameInfo editFrameInfo) {
        setBackground(null);
        setGravity(Gravity.TOP);
        setText(editFrameInfo.getmContent());
        fillEffects(editFrameInfo.getEffects());
        setSelection(length());
        setEditFrameInfo(editFrameInfo);
    }

    private void setSpan(boolean isBold, boolean isItalic, boolean isUnderline, boolean isStrike, boolean isHeight, String content, float fontSize) {
        if (TextUtils.isEmpty(content))
            return;
        SpannableString mSpannable = new SpannableString(content);
        mSpannable.setSpan(new AbsoluteSizeSpan(PixelUtil.sp2px(fontSize,context)), 0, content.length(), Spanned.SPAN_INCLUSIVE_INCLUSIVE);//不设置字号，当前字号会被覆盖为默认字号
        if (isBold && isItalic)
            mSpannable.setSpan(new StyleSpan(Typeface.BOLD_ITALIC), 0, content.length(), Spanned.SPAN_INCLUSIVE_INCLUSIVE);
        else if (isBold && !isItalic)
            mSpannable.setSpan(new StyleSpan(Typeface.BOLD), 0, content.length(), Spanned.SPAN_INCLUSIVE_INCLUSIVE);
        else if (!isBold && isItalic)
            mSpannable.setSpan(new StyleSpan(Typeface.ITALIC), 0, content.length(), Spanned.SPAN_INCLUSIVE_INCLUSIVE);

        if (isUnderline)
            mSpannable.setSpan(new UnderlineSpan(), 0, content.length(), Spanned.SPAN_INCLUSIVE_INCLUSIVE);
        if (isStrike)
            mSpannable.setSpan(new StrikethroughSpan(), 0, content.length(), Spanned.SPAN_INCLUSIVE_INCLUSIVE);
        if (isHeight)
            mSpannable.setSpan(new BackgroundColorSpan(Color.parseColor("#FFF499")), 0, content.length(), Spanned.SPAN_INCLUSIVE_INCLUSIVE);

        setText(mSpannable);
    }

    public void switchHighLight(boolean highLight) {
        if (highLight) {
            getEditableText().setSpan(new BackgroundColorSpan(Color.parseColor("#FFF499")), 0, length(), Spanned.SPAN_INCLUSIVE_INCLUSIVE);
        } else {
            BackgroundColorSpan[] backgroundColorSpans = getEditableText().getSpans(0, length(), BackgroundColorSpan.class);
            for (BackgroundColorSpan backgroundColorSpan : backgroundColorSpans) {
                getEditableText().removeSpan(backgroundColorSpan);
            }
        }
        effects.setHighLight(highLight);
    }

    public void switchMiddleLine(boolean middleLine) {
        if (middleLine) {
            getEditableText().setSpan(new StrikethroughSpan(), 0, length(), Spanned.SPAN_INCLUSIVE_INCLUSIVE);
        } else {
            StrikethroughSpan[] strikethroughSpans = getEditableText().getSpans(0, length(), StrikethroughSpan.class);
            for (StrikethroughSpan strikethroughSpan : strikethroughSpans) {
                getEditableText().removeSpan(strikethroughSpan);
            }
        }
        effects.setMiddleLine(middleLine);
    }

    public void switchBottomLine(boolean bottomLine) {
        if (bottomLine) {
            getEditableText().setSpan(new UnderlineSpan(), 0, length(), Spanned.SPAN_INCLUSIVE_INCLUSIVE);
        } else {
            UnderlineSpan[] underlineSpans = getEditableText().getSpans(0, length(), UnderlineSpan.class);
            for (UnderlineSpan underlineSpan : underlineSpans) {
                getEditableText().removeSpan(underlineSpan);
            }
        }
        effects.setBottomLine(bottomLine);
    }

    public void switchStyle(boolean italic, boolean bold) {
        if (italic && !bold) {
            StyleSpan[] styleSpans = getEditableText().getSpans(0, length(), StyleSpan.class);
            for (StyleSpan styleSpan : styleSpans) {
                getEditableText().removeSpan(styleSpan);
            }
            getEditableText().setSpan(new StyleSpan(Typeface.ITALIC), 0, length(), Spanned.SPAN_INCLUSIVE_INCLUSIVE);
        } else if (!italic && bold) {
            StyleSpan[] styleSpans = getEditableText().getSpans(0, length(), StyleSpan.class);
            for (StyleSpan styleSpan : styleSpans) {
                getEditableText().removeSpan(styleSpan);
            }
            getEditableText().setSpan(new StyleSpan(Typeface.BOLD), 0, length(), Spanned.SPAN_INCLUSIVE_INCLUSIVE);
        } else if (italic && bold) {
            StyleSpan[] styleSpans = getEditableText().getSpans(0, length(), StyleSpan.class);
            for (StyleSpan styleSpan : styleSpans) {
                getEditableText().removeSpan(styleSpan);
            }
            getEditableText().setSpan(new StyleSpan(Typeface.BOLD_ITALIC), 0, length(), Spanned.SPAN_INCLUSIVE_INCLUSIVE);
        } else {
            StyleSpan[] styleSpans = getEditableText().getSpans(0, length(), StyleSpan.class);
            for (StyleSpan styleSpan : styleSpans) {
                getEditableText().removeSpan(styleSpan);
            }
        }
        effects.setItalic(italic);
        effects.setBold(bold);
    }

    public void switchTitleMode(boolean isTitleMode) {
        if (isTitleMode) {
            setLineSpacing(0, 2.0f);
        } else {
            setLineSpacing(0, 1.5f);
        }
        effects.setTitleMode(isTitleMode);
    }

//    public void setFont(int fontId) {
//        setTypeface(FontUtils.getTypeFace(fontId));
//        effects.setFontId(fontId);
//    }

    public int getType() {
        return type;
    }

    public void setType(int type) {
        this.type = type;
    }

    public SpansInfo getSizeInfo() {
        return sizeInfo;
    }

    public void setSizeInfo(SpansInfo sizeInfo) {
        this.sizeInfo = sizeInfo;
    }

    public SpansInfo getColorInfo() {
        return colorInfo;
    }

    public void setColorInfo(SpansInfo colorInfo) {
        this.colorInfo = colorInfo;
    }

    public SpansInfo getStyleInfo() {
        return styleInfo;
    }

    public void setStyleInfo(SpansInfo styleInfo) {
        this.styleInfo = styleInfo;
    }

    public List<SpansInfo> getSpansInfoList() {
        if (Utils.isListEmpty(spansInfoList) && length() > 0) {//兼容老草稿
            spansInfoList = new ArrayList<>();
            for (int i = 0; i < length(); i++) {
                SpansInfo spansInfo = new SpansInfo();
                spansInfoList.add(spansInfo);
            }
        }
        return spansInfoList;
    }

    public void setSpansInfoList(List<SpansInfo> spansInfoList) {
        this.spansInfoList = spansInfoList;
    }

    private TextAutoHeight textAutoHeight;

    public interface TextAutoHeight {
        void refreshHeight(int textHeight);
    }

    public void setTextAutoHeight(TextAutoHeight textAutoHeight) {
        this.textAutoHeight = textAutoHeight;
    }

    public float getmDefaultTextSize() {
        return mDefaultTextSize;
    }

    public void setmDefaultTextSize(float mDefaultTextSize) {
        this.mDefaultTextSize = mDefaultTextSize;
    }

    public float getmTotalTextSize() {
        return mTotalTextSize;
    }

    public void setmTotalTextSize(float mTotalTextSize) {
        this.mTotalTextSize = mTotalTextSize;
    }

    public boolean isAdd() {
        return isAdd;
    }

    public void setAdd(boolean add) {
        isAdd = add;
    }

    public int getmLastTotalStyle() {
        return mLastTotalStyle;
    }

    public void setmLastTotalStyle(int mLastTotalStyle) {
        this.mLastTotalStyle = mLastTotalStyle;
    }

    public float getOriRatio() {
        return oriRatio;
    }

    public void setOriRatio(float oriRatio) {
        this.oriRatio = oriRatio;
    }

    public float getPerSizeWidth() {
        return perSizeWidth;
    }

    public void setPerSizeWidth(float perSizeWidth) {
        this.perSizeWidth = perSizeWidth;
    }

    public boolean isBubble() {
        return isBubble;
    }

    public void setBubble(boolean bubble) {
        isBubble = bubble;
    }

    public BubbleInfo getBubbleInfo() {
        return bubbleInfo;
    }

    public void setBubbleInfo(BubbleInfo bubbleInfo) {
        this.bubbleInfo = bubbleInfo;
    }

    public float getrTop() {
        return rTop;
    }

    public void setrTop(float rTop) {
        this.rTop = rTop;
    }

    public float getrBottom() {
        return rBottom;
    }

    public void setrBottom(float rBottom) {
        this.rBottom = rBottom;
    }

    public float getrLeft() {
        return rLeft;
    }

    public void setrLeft(float rLeft) {
        this.rLeft = rLeft;
    }

    public float getrRight() {
        return rRight;
    }

    public void setrRight(float rRight) {
        this.rRight = rRight;
    }

    public boolean isCopy() {
        return isCopy;
    }

    public void setCopy(boolean copy) {
        isCopy = copy;
    }

    public boolean isOpenAutoFit() {
        return isOpenAutoFit;
    }

    public void setOpenAutoFit(boolean openAutoFit) {
        isOpenAutoFit = openAutoFit;
    }

    public TextTypeBean getTextTypeBean() {
        return textTypeBean;
    }

    public void setTextTypeBean(TextTypeBean textTypeBean) {
        this.textTypeBean = textTypeBean;
    }

    /**
     * 生成EditFrameInfo
     */
    public EditFrameInfo generateEditFrameInfo() {
        EditFrameInfo editFrameInfo = new EditFrameInfo();
        editFrameInfo.setmViewType(2);
        if (getText() != null
                && !TextUtils.isEmpty(getText().toString())
                && !TextUtils.isEmpty(getText().toString().trim())) {
            editFrameInfo.setmContent(getText().toString());
        }
        editFrameInfo.setEffects(getEffects());
        setEditFrameInfo(editFrameInfo);
        return editFrameInfo;
    }

    /**
     * 将文本效果拼接到传入的EditFrameInfo中
     */
    public EditFrameInfo appendEditFrameInfo(EditFrameInfo editFrameInfo) {
        editFrameInfo.setmViewType(2);
        if (getText() != null
                && !TextUtils.isEmpty(getText().toString())
                && !TextUtils.isEmpty(getText().toString().trim())) {
            editFrameInfo.setmContent(getText().toString());
        }
        editFrameInfo.setEffects(getEffects());
        return editFrameInfo;
    }

    public EditFrameInfo getEditFrameInfo() {
        return editFrameInfo;
    }

    public void setEditFrameInfo(EditFrameInfo editFrameInfo) {
        this.editFrameInfo = editFrameInfo;
    }

    public int getVersionCode() {
        return versionCode;
    }

    public void setVersionCode(int versionCode) {
        this.versionCode = versionCode;
    }

    public Effects getEffects() {
        return effects;
    }

    public void setEffects(Effects effects) {
        this.effects = effects;
    }

    public int getFontId() {
        return fontId;
    }

    public void setFontId(int fontId) {
        this.fontId = fontId;
    }

    /**
     * @return 初始Effects对象
     */
    public Effects getDefaultEffects() {
        Effects effects = new Effects();
        effects.setTextColor(Color.parseColor("#3f00ff"));
        effects.setTextSize(18);
        effects.setFontId(R.attr.font);
        return effects;
    }

    /**
     * 填充文字效果
     *
     * @param effects 文字效果对象
     */
    public void fillEffects(Effects effects) {
        if(effects == null){
            return;
        }
        switchBottomLine(effects.isBottomLine());
        switchHighLight(effects.isHighLight());
        switchMiddleLine(effects.isMiddleLine());
        switchStyle(effects.isItalic(), effects.isBold());

        //setFont(effects.getFontId());

        setTextSize(effects.getTextSize());
        setTextColor(effects.getTextColor());
        switchTitleMode(effects.isTitleMode());
    }

    public boolean isPreview() {
        return isPreview;
    }

    public void setPreview(boolean preview) {
        isPreview = preview;
        if (isPreview) {
            setFocusable(false);
            setFocusableInTouchMode(false);
        }
    }
}

