package com.yilesoft.app.textimage.moveedit;


import android.app.Activity;
import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Point;
import android.graphics.PointF;
import android.graphics.Rect;
import android.graphics.Typeface;
import androidx.annotation.Nullable;
import androidx.annotation.RequiresApi;

import android.os.Build;
import android.util.AttributeSet;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;
import com.share.cool.PixelUtil;
import com.share.cool.ScreenShot;
import com.yilesoft.app.textimage.R;


public class MoveLayout extends RelativeLayout implements View.OnTouchListener {

    private boolean isBorder = true;

    private int dragDirection = 0;
    private static final int TOP = 0x15;
    private static final int LEFT = 0x16;
    private static final int BOTTOM = 0x17;
    private static final int RIGHT = 0x18;
    private static final int LEFT_TOP = 0x11;
    private static final int RIGHT_TOP = 0x12;
    private static final int LEFT_BOTTOM = 0x13;
    private static final int RIGHT_BOTTOM = 0x14;
    private static final int CENTER = 0x19;


    private int lastX;
    private int lastY;
    private float lastTrueX;
    private float lastTrueY;
    private int oriX,oriY;

    private int screenWidth;
    private int screenHeight;

    private int oriLeft;
    private int oriRight;
    private int oriTop;
    private int oriBottom;
    private int oriWidth, oriHeight;
    private int initWidth, initHeight;

    /**
     * 标示此类的每个实例的id
     */
    private int identity = 0;

    /**
     * 保存此布局所显示的Image的本地路径
     */
    private String imgLocalPath;
    /**
     * 保存此布局所显示的Image的服务器地址
     */
    private String imgUrlPath;

    /**
     * 保存此布局锁展示的图标id
     */
    private int imgLocalId;//没用上
    private String imgLogcalBitmap;
    /**
     * 图片类型(图片或图标)
     */
    private int imgType;

    /**
     * 保存气泡内边距 及背景图片本地路径
     */
    private BubbleInfo bubbleInfo;
    private String editBg;

    /**
     * 触控区域设定
     */
    private int touchAreaLength = 60;

    private int minWidth = 180;
    private int minHeight = 120;
    private final int minDragWidth = 180;
    private final int minDragHeight = 90;
    private static final String TAG = MoveLayout.class.getSimpleName();

    private float ratio = DEFAULT_RATIO;
    public final static float DEFAULT_RATIO = 1f;

    private boolean isBubble = false;
    private CustomsEditText customsEditText;
    private int locationX;
    private int locationY;
    private Context mContext;
    private int childViewType;
    private int lastWidth = 0;
    private int totalDw = 0;//累积宽度差
    //存储四个角的宽高
    private int currentCornerWidth = 0;
    private int currentCornerHeight = 0;
    private boolean hasInitTouchEvent = false;//四个角的触摸监听是否设置
    private int singleLineMinWidth = 0;//单行最小宽度
    private boolean noMove = false;
    //四个角的序号
    public static final int DELETE = 1;
    public static final int COPY = 2;
    public static final int ROTATION_SCALE = 3;
    public static final int SIZE_CHANGE = 4;

    private float lastSpacing,currentSpacing;
    private float MIN_SCALE = 0.2f;//最小缩放比例
    private float MAX_SCALE = 5.0f;//最大缩放比例

    public MoveLayout(Context context, float ratio) {
        super(context);
        this.ratio = ratio;
        init(context);

    }

    public MoveLayout(Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
        init(context);
    }

    public MoveLayout(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        init(context);

    }



    private void init(Context context) {
        mContext = context;
//绑定xml
        inflate(context,R.layout.drag_sub_view,this);
        screenHeight = PixelUtil.getScreenWH(context)[1];//getResources().getDisplayMetrics().heightPixels - 40;
        screenWidth = PixelUtil.getScreenWH(context)[0];// getResources().getDisplayMetrics().widthPixels;
        //添加默认的edittext
        CustomsEditText customsEditText=new CustomsEditText(context);
        customsEditText.setText("点此输入文字");
        RelativeLayout rlt = (RelativeLayout) getChildAt(0);
        ((LinearLayout) rlt.getChildAt(0)).addView(customsEditText);
    }

    @Override
    protected void onSizeChanged(int w, int h, int oldw, int oldh) {
        super.onSizeChanged(w, h, oldw, oldh);
        if(oldw == 0 && w != 0){
            MIN_SCALE = (float) minHeight/h;
        }
    }

    public void setViewWidthHeight(int width, int height) {
        screenWidth = width;
        screenHeight = height;
    }

    public void setMinHeight(int height) {
        minHeight = height;
        if (minHeight < touchAreaLength * 2) {
            minHeight = touchAreaLength * 2;
        }
    }

    public void setMinWidth(int width) {
        minWidth = width;
        if (minWidth < touchAreaLength * 3) {
            minWidth = touchAreaLength * 3;
        }
    }

    public String getImgUrlPath() {
        return imgUrlPath;
    }

    public void setImgUrlPath(String imgUrlPath) {
        this.imgUrlPath = imgUrlPath;
    }

    public BubbleInfo getBubbleInfo() {
        return bubbleInfo;
    }

    public void setBubbleInfo(BubbleInfo bubbleInfo) {
        this.bubbleInfo = bubbleInfo;
    }
    public String getEditBg() {
        return editBg;
    }

    public void setEditBg(String editBg) {
        this.editBg = editBg;
    }

    public boolean isBorder() {
        return isBorder;
    }

    public void setBorder(boolean border) {
        isBorder = border;
    }

    private int mDeleteHeight = 0;
    private int mDeleteWidth = 0;
    private boolean isInDeleteArea = false;

    public void setDeleteWidthHeight(int width, int height) {
        mDeleteWidth = screenWidth - width;
        mDeleteHeight = height;
    }

    @Override
    protected void onDraw(Canvas canvas) {
        super.onDraw(canvas);

        if (!hasInitTouchEvent) {
            RelativeLayout rlt = (RelativeLayout) getChildAt(0);
            View view = ((LinearLayout) rlt.getChildAt(0)).getChildAt(0);
            rlt.getChildAt(DELETE).setOnTouchListener(this);
            rlt.getChildAt(COPY).setOnTouchListener(this);
            rlt.getChildAt(ROTATION_SCALE).setOnTouchListener(this);
            rlt.getChildAt(SIZE_CHANGE).setOnTouchListener(this);

            if (view instanceof ImageView) {
                rlt.getChildAt(ROTATION_SCALE).setBackgroundResource(R.drawable.st_rotate_icon);
                rlt.getChildAt(COPY).setVisibility(GONE);
            } else if (view instanceof CustomsEditText) {
                rlt.getChildAt(SIZE_CHANGE).setBackgroundResource(R.drawable.st_rotate_icon);
            }
            hasInitTouchEvent = true;
        }
    }

    // 属性变量
    private float translationX; // 移动X
    private float translationY; // 移动Y
    private float scale = 1; // 伸缩比例
    private float rotation; // 旋转角度

    // 移动过程中临时变量
    private float actionX;
    private float actionY;
    private float spacing;
    private float degree;
    private int moveType; // 0=未选择，1=拖动，2=缩放

    private boolean isImage = false;
    private boolean isCustomEdit = false;
    private boolean isDrag = false;
    private float textRotationDegree = 0;//文本框旋转角度

    @Override
    public boolean onTouch(View v, MotionEvent event) {
        isDrag = true;
        int type = 0;
        RelativeLayout rl = (RelativeLayout) getChildAt(0);
        View view = ((LinearLayout) rl.getChildAt(0)).getChildAt(0);
        if (view instanceof ImageView) {
            type = 0;
        } else if (view instanceof CustomsEditText) {
            type = 1;
        }
        int vId = v.getId();
        switch (vId) {
            case R.id.v_lt:
                if(onCornersClickListener != null){
                    onCornersClickListener.onDelete(this);
                }
//                dragDirection = LEFT_TOP;
                break;
            case R.id.v_lb:
                if(onCornersClickListener != null){
                    onCornersClickListener.onCopy(this);
                }
//                dragDirection = LEFT_BOTTOM;
                break;
            case R.id.v_rt:
                if(type == 1){
                    dragDirection = RIGHT_TOP;
                }else{
                    if(onCornersClickListener != null){
                        onCornersClickListener.onCopy(this);
                    }
                }
//                dragDirection = RIGHT_TOP;
                break;
            case R.id.v_rb:
                //图片旋转缩放 文字宽度缩放
                dragDirection = RIGHT_BOTTOM;
                break;
            default:
                dragDirection = CENTER;
                break;
        }
        return false;
    }

    @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
    @Override
    public boolean onTouchEvent(MotionEvent event) {
        if(noMove){
            return super.onTouchEvent(event);
        }
        isImage = false;
        isCustomEdit = false;
        RelativeLayout rl = (RelativeLayout) getChildAt(0);
        View view = ((LinearLayout) rl.getChildAt(0)).getChildAt(0);
        if (view instanceof ImageView) {
            isImage = true;
        } else if (view instanceof CustomsEditText) {
            isCustomEdit = true;
        }
        if (!isImage) {
            int action = event.getAction();
            switch (action) {
                case MotionEvent.ACTION_DOWN:
                    if (mOnTouchListener != null) {
                        mOnTouchListener.onTouch(this);
                    }
                    setBorder(true);
                    if (view instanceof CustomsEditText) {
                        customsEditText = (CustomsEditText) view;
                    }
                    oriLeft = getLeft();
                    oriRight = getRight();
                    oriTop = getTop();
                    oriBottom = getBottom();

                    lastY = (int) event.getRawY();
                    lastX = (int) event.getRawX();
                    oriX = (int) event.getRawX();
                    oriY = (int) event.getRawY();
                    int x = (int) event.getX();
                    int y = (int) event.getY();
                    if (!isDrag) {
                        dragDirection = CENTER;
                    } else {
                        setMinWidth(minDragWidth);
                        setMinHeight(minDragHeight);
                        if (isCustomEdit) {
                            customsEditText.setMinWidth(minDragWidth - 30);
                            customsEditText.setMinHeight(minDragHeight - 30);
                        }
                    }
                    break;
                case MotionEvent.ACTION_UP:
                    if (mOnTouchListener != null) {
                        mOnTouchListener.onActionUp();
                    }
                    isDrag = false;
                    requestLayout();
                    break;
                case MotionEvent.ACTION_MOVE:
                    updateLocation();
                    if (mOnTouchListener != null) {
                        mOnTouchListener.onMoveEvent(this);
                    }
                    int tempRawX = (int) event.getRawX();
                    int tempRawY = (int) event.getRawY();

                    int dx = tempRawX - lastX;
                    int dy = tempRawY - lastY;
                    lastX = tempRawX;
                    lastY = tempRawY;
                    switch (dragDirection) {
                        case CENTER:
                            center(dx, dy);
                            break;
                        case LEFT_BOTTOM:
                            if (isImage) {
                                if (dx != 0 && dy != 0) {
                                    left(dx);
                                    bottom(dy);
                                }
                            } else {
                                if (isCustomEdit) {
                                    left(dx);
                                }
                            }
                            break;
                        case LEFT_TOP:
                            if (isImage) {
                                if (dx != 0 && dy != 0) {
                                    left(dx);
                                    top(dy);
                                }
                            } else {
                                if (isCustomEdit) {
                                    left(dx);
                                }
                            }
                            break;
                        case RIGHT_BOTTOM:
                            if (isImage) {
                                if (dx != 0 && dy != 0) {
                                    right(dx);
                                    bottom(dy);
                                }
                            } else {
                                if (isCustomEdit) {
                                    right(dx);
//                                bottom(dx);
                                }
                            }

                            break;
                        case RIGHT_TOP:
                            if (isImage) {
                                if (dx != 0 && dy != 0) {
                                    right(dx);
                                    top(dy);
                                }
                            } else {
                                if (isCustomEdit) {//旋转
                                    Point cen = new Point(oriLeft+(oriRight-oriLeft)/2,oriTop+(oriBottom-oriTop)/2);
                                    Point first = new Point(oriX,oriY);
                                    Point second = new Point(tempRawX,tempRawY);
                                    textRotationDegree += angle(cen,first,second);
                                    setRotation(textRotationDegree);
                                    oriX = tempRawX;
                                    oriY = tempRawY;
                                }
                            }

                            break;
                    }
                    LayoutParams lp;
                    if (isImage) {
                        lp = new LayoutParams(oriRight - oriLeft, (int) ((oriRight - oriLeft) / ratio));
                    } else {
                        if (customsEditText != null && isCustomEdit) {
                            int height;
                            if(isBubble){
                                if(dragDirection == CENTER){
                                    height = oriBottom - oriTop;
                                }else{
                                    height = customsEditText.getTextHeight();
                                }
                            }else{
                                height = customsEditText.getTextHeight();
                            }
                            int width = oriRight - oriLeft;
                            if (width < singleLineMinWidth) {
                                width = singleLineMinWidth;
                            }
                            lp = new LayoutParams(width, height);
                        } else {
                            lp = new LayoutParams(oriRight - oriLeft, oriBottom - oriTop);
                        }
                    }
                    lp.setMargins(oriLeft, oriTop, 0, 0);
                    setLayoutParams(lp);
                    if(isBubble && dragDirection != CENTER){//平移不调整间距
                        //动态调整间距
                        int width = customsEditText.getWidth();
                        int height = customsEditText.getHeight();
                        int paddingLeft = (int) (customsEditText.getrLeft()*width/* 0.7*/);
                        int paddingTop = (int) (customsEditText.getrTop()*height/* 0.7*/);
                        int paddingRight = (int)(customsEditText.getrRight()*width/* 0.7*/);
                        int paddingBottom = (int)(customsEditText.getrBottom()*height/* 0.7*/);
                        customsEditText.setPadding(paddingLeft,paddingTop,paddingRight,paddingBottom);
                    }

                    int dw = (oriRight - oriLeft) - lastWidth;
                    totalDw += dw;
                    lastWidth = oriRight - oriLeft;
                    break;
            }
            return super.onTouchEvent(event);
        } else {
            switch (event.getAction() & MotionEvent.ACTION_MASK) {
                case MotionEvent.ACTION_DOWN:
                    if (mOnTouchListener != null) {
                        mOnTouchListener.onTouch(this);
                    }
                    setBorder(true);
                    oriLeft = getLeft();
                    oriRight = getRight();
                    oriTop = getTop();
                    oriBottom = getBottom();
                    lastTrueY = event.getRawY();
                    lastTrueX = event.getRawX();
                    oriX = (int) event.getRawX();
                    oriY = (int) event.getRawY();
                    lastSpacing = getSpacingRelateMidPoint(oriX,oriY);
                    textRotationDegree = getRotation();
                    scale = getScaleX();
                    if (initWidth == 0) {
                        initWidth = oriRight - oriLeft;
                        if (initWidth <= 0) {
                            initWidth = minWidth;
                        }
                    }
                    if (initHeight == 0) {
                        initHeight = oriBottom - oriTop;
                        if (initHeight <= 0) {
                            initHeight = minHeight;
                        }
                    }
                    LogUtil.d(TAG + " initWidth = " + initWidth + " ,initHeight = " + initHeight);
                    moveType = 1;
                    actionX = event.getRawX();
                    actionY = event.getRawY();
                    break;
                case MotionEvent.ACTION_POINTER_DOWN:
                    moveType = 2;
                    spacing = getSpacing(event);
                    degree = getDegree(event);
                    break;
                case MotionEvent.ACTION_MOVE:
                    if (mOnTouchListener != null) {
                        mOnTouchListener.onMoveEvent(this);
                    }
                    updateLocation();//更新图标图片可见矩形
                    float tempRawX = event.getRawX();
                    float tempRawY = event.getRawY();

                    float dx = tempRawX - lastTrueX;
                    float dy = tempRawY - lastTrueY;
                    lastTrueX = tempRawX;
                    lastTrueY = tempRawY;
                    if (event.getPointerCount() == 1 && dragDirection == RIGHT_BOTTOM) {
                        Point cen = new Point(oriLeft+(oriRight-oriLeft)/2,oriTop+(oriBottom-oriTop)/2);
                        Point first = new Point(oriX,oriY);
                        Point second = new Point((int) tempRawX,(int) tempRawY);

                        //旋转
                        textRotationDegree += angle(cen,first,second);
                        setRotation(textRotationDegree);
                        //缩放
                        float tempScale = 1.0f;
                        lastSpacing = getSpacing(cen,first);
                        currentSpacing = getSpacing(cen,second);
                        tempScale = currentSpacing/lastSpacing;
                        //想增加拖动缩放的灵敏度，将0.02增大即可
                        if (tempScale < 1) {
                            tempScale -= 0.008;
                        } else if (tempScale > 1) {
                            tempScale += 0.008;
                        }
                        scale = scale * tempScale;
                        if(scale >= MAX_SCALE){
                            scale = MAX_SCALE;
                        }else if(scale <= MIN_SCALE){
                            scale = MIN_SCALE;
                        }
                        setScaleX(scale);
                        setScaleY(scale);

                        oriX = (int) tempRawX;
                        oriY = (int) tempRawY;
                        lastSpacing = currentSpacing;
                        resetCornerSize();
                        break;
                    }
                    if (moveType == 1) {
                        translationX = getTranslationX();
                        translationY = getTranslationY();
                        if (isTranstX(event.getRawX()))
                            translationX = translationX + event.getRawX() - actionX;
                        if (isTranstY(event.getRawY()))
                            translationY = translationY + event.getRawY() - actionY;
                        setTranslationX(translationX);
                        setTranslationY(translationY);
                        actionX = event.getRawX();
                        actionY = event.getRawY();
                    } else if (moveType == 2) {
                        scale = scale * getSpacing(event) / spacing;
                        setScaleX(scale);
                        setScaleY(scale);
                        rotation = rotation + getDegree(event) - degree;
                        if (rotation > 360) {
                            rotation = rotation - 360;
                        }
                        if (rotation < -360) {
                            rotation = rotation + 360;
                        }
                        setRotation(rotation);
                        resetCornerSize();
                    }
                    break;
                case MotionEvent.ACTION_UP:
                case MotionEvent.ACTION_POINTER_UP:
                    if (mOnTouchListener != null) {
                        mOnTouchListener.onActionUp();
                    }
                    moveType = 0;
                    dragDirection = CENTER;
                    requestLayout();
            }
            return super.onTouchEvent(event);
        }
    }

    public float angle(Point cen, Point first, Point second)
    {
        float dx1, dx2, dy1, dy2;

        dx1 = first.x - cen.x;
        dy1 = first.y - cen.y;
        dx2 = second.x - cen.x;
        dy2 = second.y - cen.y;

        // 计算三边的平方
        float ab2 = (second.x - first.x) * (second.x - first.x) + (second.y - first.y) * (second.y - first.y);
        float oa2 = dx1*dx1 + dy1*dy1;
        float ob2 = dx2 * dx2 + dy2 *dy2;

        // 根据两向量的叉乘来判断顺逆时针
        boolean isClockwise = ((first.x - cen.x) * (second.y - cen.y) - (first.y - cen.y) * (second.x - cen.x)) > 0;

        // 根据余弦定理计算旋转角的余弦值
        double cosDegree = (oa2 + ob2 - ab2) / (2 * Math.sqrt(oa2) * Math.sqrt(ob2));

        // 异常处理，因为算出来会有误差绝对值可能会超过一，所以需要处理一下
        if (cosDegree > 1) {
            cosDegree = 1;
        } else if (cosDegree < -1) {
            cosDegree = -1;
        }

        // 计算弧度
        double radian = Math.acos(cosDegree);

        // 计算旋转过的角度，顺时针为正，逆时针为负
        return (float) (isClockwise ? Math.toDegrees(radian) : -Math.toDegrees(radian));

    }

    /**
     * 计算单个向量的角度
     *
     * @param vector 需要计算角度的向量
     * @return 返回手指旋转过的角度
     */
    private float calculateDegree(PointF vector) {
        float degree = (float) Math.atan2(vector.y, vector.x);
        return (float) Math.toDegrees(degree);
    }

    View lt, lb, rt, rb;
    RelativeLayout relativeLayout;

    //重置四个角的宽高
    private void resetCornerSize() {
        if (relativeLayout == null) {
            relativeLayout = (RelativeLayout) getChildAt(0);
        }
        if (lt == null)
            lt = relativeLayout.getChildAt(1);
        if (lb == null)
            lb = relativeLayout.getChildAt(2);
        if (rt == null)
            rt = relativeLayout.getChildAt(3);
        if (rb == null)
            rb = relativeLayout.getChildAt(4);
        lt.setLayoutParams(setLayoutParams(lt));
        lb.setLayoutParams(setLayoutParams(lb));
        rt.setLayoutParams(setLayoutParams(rt));
        rb.setLayoutParams(setLayoutParams(rb));
    }

    private LayoutParams setLayoutParams(View view) {
        LayoutParams ltparams = (LayoutParams) view.getLayoutParams();
        currentCornerWidth = (int) (PixelUtil.dp2Pixel(19,mContext) / scale);
        currentCornerHeight = (int) (PixelUtil.dp2Pixel(19,mContext) / scale);
        ltparams.width = currentCornerWidth;
        ltparams.height = currentCornerHeight;
        return ltparams;
    }

    Rect mRect = null;

    private void updateLocation() {
        Rect rect = new Rect();
        if (mContext instanceof Activity) {
            getGlobalVisibleRect(rect);
            mRect = rect;
            setLocationX(rect.left);
            setLocationY(rect.top - PixelUtil.getStatusBarHeight(((Activity) getContext())));
            int height = PixelUtil.getScreenWH((Activity) mContext)[1];
        } else {
            LogUtil.e(TAG + " get location fail : context is not instance of Activity");
        }
    }

    private boolean isTranstX(float rawX) {
        float effset = rawX - actionX;
        if ((mRect.left > 0 || (mRect.left == 0 && effset > 0)) && (mRect.right < screenWidth || (mRect.right == screenWidth && effset < 0)))
            return true;
        return false;
    }

    private boolean isTranstY(float rawY) {
        float effset = rawY - actionY;
        int statusHeight = PixelUtil.getStatusBarHeight(((Activity) getContext()));
        if ((mRect.top - statusHeight > 0 || (mRect.top - statusHeight == 0 && effset > 0)) && (mRect.bottom < screenHeight + statusHeight || ((mRect.bottom == screenHeight + statusHeight) && effset < 0)))
            return true;
        return false;
    }

    // 触碰两点间距离
    private float getSpacing(MotionEvent event) {
        //通过三角函数得到两点间的距离
        float x = event.getX(0) - event.getX(1);
        float y = event.getY(0) - event.getY(1);
        return (float) Math.sqrt(x * x + y * y);
    }

    // 触碰两点间距离
    private float getSpacing(Point start,Point end) {
        //通过三角函数得到两点间的距离
        float x = end.x - start.x;
        float y = end.y - start.y;
        return (float) Math.sqrt(x * x + y * y);
    }

    // 获得某点相对于中点的距离
    private float getSpacingRelateMidPoint(int pointX,int pointY) {
        //通过三角函数得到两点间的距离
        Point midPoint = new Point(oriLeft+(oriRight-oriLeft),oriTop+(oriBottom-oriTop));
        float x = pointX - midPoint.x;
        float y = pointY - midPoint.y;
        return (float) Math.sqrt(x * x + y * y);
    }

    // 取旋转角度
    private float getDegree(MotionEvent event) {
        //得到两个手指间的旋转角度
        double delta_x = event.getX(0) - event.getX(1);
        double delta_y = event.getY(0) - event.getY(1);
        double radians = Math.atan2(delta_y, delta_x);
        return (float) Math.toDegrees(radians);
    }

    /**
     * 触摸点为中心->>移动
     */
    private void center(int dx, int dy) {
//        isTopLimit = false;
//        isBottomLimit = false;
        int left = getLeft() + dx;
        int top = getTop() + dy;
        int right = getRight() + dx;
        int bottom = getBottom() + dy;

        if (left < 0) {
            left = 0;
            right = left + getWidth();
        }
        if (right > screenWidth) {
            right = screenWidth;
            left = right - getWidth();
        }
        if (top < 0) {
            top = 0;
            bottom = top + getHeight();
//            isTopLimit = true;
        }
        if (bottom > screenHeight) {
            bottom = screenHeight;
            top = bottom - getHeight();
//            isBottomLimit = true;
        }

        oriLeft = left;
        oriTop = top;
        oriRight = right;
        oriBottom = bottom;
    }

    /**
     * 触摸点为上边缘
     */
    private void top(int dy) {
//        isTopLimit = false;
//        isBottomLimit = false;

        int lastTop = oriTop;
        oriTop += dy;
        int newTop = oriTop;
        Log.e("新旧长度：", lastTop + "," + newTop);
        if (isImage) {
            oriLeft += dy / 2;
            oriRight -= dy / 2;

            if (oriLeft < 0) {
                oriLeft = 0;
            }
            if (oriRight - oriLeft < minWidth) {
                oriLeft = oriRight - minWidth;
            }

            if (oriRight > screenWidth) {
                oriRight = screenWidth;
            }
            if (oriRight - oriLeft < minWidth) {
                oriRight = oriLeft + minWidth;
            }
        }
        if (oriTop < 0) {
            oriTop = 0;
//            isTopLimit = true;
        }
        if (oriBottom - oriTop < minHeight) {
            oriTop = oriBottom - minHeight;
        }
    }

    /**
     * 触摸点为下边缘
     */
    private void bottom(int dy) {
//        isTopLimit = false;
//        isBottomLimit = false;
        oriBottom += dy;
        if (isImage) {
            oriLeft -= dy / 2;
            oriRight += dy / 2;

            if (oriLeft < 0) {
                oriLeft = 0;
            }
            if (oriRight - oriLeft < minWidth) {
                oriLeft = oriRight - minWidth;
            }

            if (oriRight > screenWidth) {
                oriRight = screenWidth;
            }
            if (oriRight - oriLeft < minWidth) {
                oriRight = oriLeft + minWidth;
            }
        }
        if (oriBottom > screenHeight) {
            oriBottom = screenHeight;
//            isBottomLimit = true;
        }
        if (oriBottom - oriTop < minHeight) {
            oriBottom = minHeight + oriTop;
        }
    }

    /**
     * 触摸点为右边缘
     */
    private void right(int dx) {
//        isTopLimit = false;
//        isBottomLimit = false;
        oriRight += dx;
        if (isImage) {
            oriTop -= dx / 2;
            oriBottom += dx / 2;

            if (oriTop < 0) {
                oriTop = 0;
//                isTopLimit = true;
            }
            if (oriBottom - oriTop < minHeight)
                oriTop = oriBottom = minHeight;

            if (oriBottom > screenHeight) {
                oriBottom = screenHeight;
//                isBottomLimit = true;
            }
            if (oriBottom - oriTop < minHeight) {
                oriBottom = minHeight + oriTop;
            }
        }
        if (oriRight > screenWidth) {
            oriRight = screenWidth;
        }
        if (oriRight - oriLeft < minWidth) {
            oriRight = oriLeft + minWidth;
        }
    }

    /**
     * 触摸点为左边缘
     */
    private void left(int dx) {
//        isTopLimit = false;
//        isBottomLimit = false;
        oriLeft += dx;
        if (isImage) {
            oriTop += dx / 2;
            oriBottom -= dx / 2;

            if (oriTop < 0) {
                oriTop = 0;
//                isTopLimit = true;
            }
            if (oriBottom - oriTop < minHeight)
                oriTop = oriBottom = minHeight;

            if (oriBottom > screenHeight) {
                oriBottom = screenHeight;
//                isBottomLimit = true;
            }
            if (oriBottom - oriTop < minHeight) {
                oriBottom = minHeight + oriTop;
            }
        }
        if (oriLeft < 0) {
            oriLeft = 0;
        }
        if (oriRight - oriLeft < minWidth) {
            oriLeft = oriRight - minWidth;
        }
    }

    private int getDirection(int x, int y) {
        int left = getLeft();
        int right = getRight();
        int bottom = getBottom();
        int top = getTop();

        if (x < touchAreaLength && y < touchAreaLength) {
            return LEFT_TOP;
        }
        if (y < touchAreaLength && right - left - x < touchAreaLength) {
            return RIGHT_TOP;
        }
        if (x < touchAreaLength && bottom - top - y < touchAreaLength) {
            return LEFT_BOTTOM;
        }
        if (right - left - x < touchAreaLength && bottom - top - y < touchAreaLength) {
            return RIGHT_BOTTOM;
        }

        if (x < touchAreaLength) {
            requestLayout();
            return LEFT;
        }
        if (y < touchAreaLength) {
            requestLayout();
            return TOP;
        }
        if (right - left - x < touchAreaLength) {
            requestLayout();
            return RIGHT;
        }
        if (bottom - top - y < touchAreaLength) {
            requestLayout();
            return BOTTOM;
        }
        return CENTER;
    }

    @Override
    protected void onFocusChanged(boolean gainFocus, int direction, Rect previouslyFocusedRect) {
        super.onFocusChanged(gainFocus, direction, previouslyFocusedRect);
        LinearLayout linearLayout1 = (LinearLayout) ((RelativeLayout) this.getChildAt(0)).getChildAt(0);
        View deleteView = ((RelativeLayout) this.getChildAt(0)).getChildAt(DELETE);
        View copyView = ((RelativeLayout) this.getChildAt(0)).getChildAt(COPY);
        View rotationView = ((RelativeLayout) this.getChildAt(0)).getChildAt(ROTATION_SCALE);
        View sizeChangeView = ((RelativeLayout) this.getChildAt(0)).getChildAt(SIZE_CHANGE);
        TextView tv_hint = ((RelativeLayout) this.getChildAt(0)).findViewById(R.id.tv_hint);

        if (gainFocus) {
            View view = linearLayout1.getChildAt(0);
            if (view instanceof CustomsEditText) {
                CustomsEditText customsEditText = (CustomsEditText) view;
                if (customsEditText.length() > 0) {
                    linearLayout1.setBackgroundResource(R.drawable.textbg_selector);
                    deleteView.setVisibility(View.VISIBLE);
                    copyView.setVisibility(View.VISIBLE);
                    rotationView.setVisibility(View.VISIBLE);
                    sizeChangeView.setVisibility(View.VISIBLE);
                    tv_hint.setVisibility(VISIBLE);
                    tv_hint.setText("双击编辑文字");
                } else {

                    customsEditText.setFocusable(true);
                    customsEditText.setFocusableInTouchMode(true);
                    customsEditText.requestFocus();
                    linearLayout1.setBackgroundResource(R.drawable.edit_bg_selector);
                    deleteView.setVisibility(View.GONE);
                    copyView.setVisibility(View.GONE);
                    rotationView.setVisibility(View.GONE);
                    sizeChangeView.setVisibility(View.GONE);
                    tv_hint.setVisibility(GONE);
                }
            } else {
                linearLayout1.setBackgroundResource(R.drawable.textbg_selector);
                deleteView.setVisibility(View.VISIBLE);
                copyView.setVisibility(View.GONE);
                rotationView.setVisibility(VISIBLE);
                rotationView.setBackgroundResource(R.drawable.edit_bg_selector);
                sizeChangeView.setVisibility(View.VISIBLE);
                tv_hint.setVisibility(GONE);
            }
        } else {
            linearLayout1.setBackgroundResource(0);
            deleteView.setVisibility(View.GONE);
            copyView.setVisibility(View.GONE);
            rotationView.setVisibility(View.GONE);
            sizeChangeView.setVisibility(View.GONE);
            tv_hint.setVisibility(GONE);
        }
    }

    public int getImgType() {
        return imgType;
    }

    public void setImgType(int imgType) {
        this.imgType = imgType;
    }

    public int getImgLocalId() {
        return imgLocalId;
    }

    public void setImgLocalId(int imgLocalId) {
        this.imgLocalId = imgLocalId;
    }

    public int getIdentity() {
        return identity;
    }

    public void setIdentity(int identity) {
        this.identity = identity;
    }

    public String getImgLocalPath() {
        return imgLocalPath;
    }

    public void setImgLocalPath(String imgLocalPath) {
        this.imgLocalPath = imgLocalPath;
    }

    private OnTouchListener mOnTouchListener;

    protected interface OnTouchListener {
        void onTouch(View view);

        void onMoveEvent(MoveLayout moveLayout);

        void onActionUp();
    }

    public void setOnTouchListener(OnTouchListener mOnTouchListener) {
        this.mOnTouchListener = mOnTouchListener;
    }

    public boolean isBubble() {
        return isBubble;
    }

    public void setBubble(boolean bubble) {
        isBubble = bubble;
    }

    public int getOriWidth() {
        return oriWidth;
    }

    public void setOriWidth(int oriWidth) {
        this.oriWidth = oriWidth;
    }

    public int getOriHeight() {
        return oriHeight;
    }

    public void setOriHeight(int oriHeight) {
        this.oriHeight = oriHeight;
    }

    public CustomsEditText getCustomsEditText() {
        return customsEditText;
    }

    public void setCustomsEditText(CustomsEditText customsEditText) {
        this.customsEditText = customsEditText;
    }

    public String getImgLogcalBitmap() {
        return imgLogcalBitmap;
    }

    public void setImgLogcalBitmap(String imgLogcalBitmap) {
        this.imgLogcalBitmap = imgLogcalBitmap;
    }

    public int getLocationX() {
        return locationX;
    }

    public void setLocationX(int locationX) {
        this.locationX = locationX;
    }

    public int getLocationY() {
        return locationY;
    }

    public void setLocationY(int locationY) {
        this.locationY = locationY;
    }

    public float getRatio() {
        return ratio;
    }

    public void setRatio(float ratio) {
        this.ratio = ratio;
    }

    public int getChildViewType() {
        return childViewType;
    }

    public void setChildViewType(int childViewType) {
        this.childViewType = childViewType;
    }

    public EditFrameInfo getEditFrameInfo() {
        //用于编辑框信息
        EditFrameInfo editFrameInfo = new EditFrameInfo();
        LinearLayout addViewLinear = (LinearLayout) ((RelativeLayout) getChildAt(0)).getChildAt(0);
        View view = ((LinearLayout) ((RelativeLayout) getChildAt(0)).getChildAt(0)).getChildAt(0);
        editFrameInfo.setmRawX((int) PixelUtil.dp2Pixel(getLeft(),mContext));
        editFrameInfo.setmRawY((int) PixelUtil.dp2Pixel(getTop(),mContext));
        editFrameInfo.setmWidth(addViewLinear.getWidth());
        editFrameInfo.setmHeight(addViewLinear.getHeight());
        editFrameInfo.setCornerWidth(currentCornerWidth);
        editFrameInfo.setCornerHeight(currentCornerHeight);
        LogUtil.d("MyEditText : " + currentCornerWidth + "," + currentCornerHeight);
        LogUtil.e("图片父容器宽高：" + addViewLinear.getWidth() + "/" + addViewLinear.getHeight());
        if (view instanceof ImageView) {
            editFrameInfo.setRotation(getRotation());
            editFrameInfo.setScaleX(getScaleX());
            editFrameInfo.setScaleY(getScaleY());
            editFrameInfo.setTransX(getTranslationX());
            editFrameInfo.setTransY(getTranslationY());
            if (getImgType() == 1) {
                editFrameInfo.setmViewType(1);
                editFrameInfo.setmLocalImgPath(getImgLocalPath());
                editFrameInfo.setmUrlImagePath(getImgUrlPath());
                //TODO 此处存的是像素 不同手机上会有偏差 等测试上边的 X Y方案可行再改
                ImageView imageView = (ImageView) addViewLinear.getChildAt(0);
                editFrameInfo.setmWidth(imageView.getWidth());
                editFrameInfo.setmHeight(imageView.getHeight());
                LogUtil.e("图片宽高：" + imageView.getWidth() + "/" + imageView.getHeight());
            }
            if (getImgType() == 3) {
                editFrameInfo.setmViewType(3);
                editFrameInfo.setmLocalImgId(getImgLocalId());
                editFrameInfo.setmLocalImgBitmap(getImgLogcalBitmap());
            }
        }
        if (view instanceof CustomsEditText) {
            CustomsEditText customsEditText = (CustomsEditText) view;
            editFrameInfo.setmViewType(2);
            editFrameInfo.setmTextColor(customsEditText.getCurrentTextColor());
            editFrameInfo.setmTextSize(PixelUtil.px2sp(customsEditText.getTextSize(),mContext));
            editFrameInfo.setmContent(customsEditText.getText().toString());
            editFrameInfo.setmBubbleInfo(getBubbleInfo());
            editFrameInfo.setEidtBg(getEditBg());
            if (customsEditText.getmLastTotalStyle() == 1) {
                editFrameInfo.setTextType(Typeface.BOLD);
            } else {
                editFrameInfo.setTextType(Typeface.NORMAL);
            }
            if (customsEditText.getSizeInfo() != null) {
                editFrameInfo.setSizeInfo(customsEditText.getSizeInfo());
            }
            if (customsEditText.getColorInfo() != null) {
                editFrameInfo.setColorInfo(customsEditText.getColorInfo());
            }
            if (customsEditText.getStyleInfo() != null) {
                editFrameInfo.setStyleInfo(customsEditText.getStyleInfo());
            }
            if (!Utils.isListEmpty(customsEditText.getSpansInfoList())) {
                editFrameInfo.setSpansInfoList(customsEditText.getSpansInfoList());
            }
        }
        return editFrameInfo;
    }

    public int getMinHeight() {
        return minHeight;
    }

    public int getMinWidth() {
        return minWidth;
    }

    public int getCurrentCornerWidth() {
        return currentCornerWidth;
    }

    public void setCurrentCornerWidth(int currentCornerWidth) {
        this.currentCornerWidth = currentCornerWidth;
    }

    public int getCurrentCornerHeight() {
        return currentCornerHeight;
    }

    public void setCurrentCornerHeight(int currentCornerHeight) {
        this.currentCornerHeight = currentCornerHeight;
    }

    public int getSingleLineMinWidth() {
        return singleLineMinWidth;
    }

    public void setSingleLineMinWidth(int singleLineMinWidth) {
        this.singleLineMinWidth = singleLineMinWidth;
    }

    public boolean isNoMove() {
        return noMove;
    }

    public void setNoMove(boolean noMove) {
        this.noMove = noMove;
    }

    public interface OnCornersClickListener{
        void onCopy(View view);
        void onDelete(View view);
    }

    private OnCornersClickListener onCornersClickListener;

    public OnCornersClickListener getOnCornersClickListener() {
        return onCornersClickListener;
    }

    public void setOnCornersClickListener(OnCornersClickListener onCornersClickListener) {
        this.onCornersClickListener = onCornersClickListener;
    }
}
